@@ -10,10 +10,10 @@ from typing import Union, Iterator
 from celery import current_app
 from celery.result import AsyncResult
 from celery.signals import before_task_publish, task_prerun
-from celery.states import FAILURE, REVOKED, PENDING, STARTED, RECEIVED, RETRY
+from celery.states import FAILURE, REVOKED, PENDING, STARTED, RECEIVED, RETRY, SUCCESS
 from celery.utils.log import get_task_logger
 from firexkit.broker import handle_broker_timeout
-from firexkit.inspect import get_task, get_active_queues
+from firexkit.inspect import get_task, get_active_queues, get_active
 from firexkit.revoke import RevokedRequests
 
 RETURN_KEYS_KEY = '__task_return_keys'
@@ -47,11 +47,16 @@ def get_task_name_from_result(result):
 
 
 def get_task_queue_from_result(result):
+    queue = ''
     try:
-        return get_task_info_from_result(result=result, key='queue')
+        while not queue and result:
+            queue = get_task_info_from_result(result=result, key='queue')
+            #  Try to get queue of parent tasks in the same chain, if current item hasn't been scheduled yet
+            result = result.parent
     except AttributeError:
         logger.exception('Task queue info not supported for this broker')
-        return ''
+
+    return queue
 
 
 def get_tasks_names_from_results(results):
@@ -188,11 +193,23 @@ def _is_worker_alive(result: AsyncResult, timeout=None, retry_delay=0.1, retries
                              f' Info: {info}, Hostname: {hostname}')
                 return True
 
-            task_info = get_task(method_args=(result.id,), destination=(hostname,), timeout=10)
+            task_id = result.id
+            task_info = get_task(method_args=(task_id,), destination=(hostname,), timeout=60)
             if task_info and any(task_info.values()):
                 return True
+            else:
+                # Try get_active, since we suspect query_task (the api used by get_task) may be broken sometimes.
+                active_tasks = get_active(destination=(hostname,), timeout=60)
+                task_list = active_tasks.get(hostname) if active_tasks else None
+                if task_list:
+                    for task in task_list:
+                        this_task_id = task.get('id')
+                        if this_task_id == task_id:
+                            return True
 
-            logger.debug(f'Task inspection for {task_name} on {hostname} returned:\n{pformat(task_info)}')
+            logger.debug(f'Task inspection for {task_name} on {hostname} with id '
+                         f'of {task_id} returned:\n{pformat(task_info)}\n'
+                         f'Active tasks:\n{pformat(active_tasks)}')
 
         elif state == PENDING or state == RETRY:
             # Check if task queue is alive
@@ -206,7 +223,7 @@ def _is_worker_alive(result: AsyncResult, timeout=None, retry_delay=0.1, retries
                 logger.debug(f'Queue "{task_queue}" for {task_name} not seen yet; assuming task is alive.')
                 return True
 
-            queues = get_active_queues()
+            queues = get_active_queues(timeout=60)
             active_queues = {queue['name'] for node in queues.values() for queue in node} if queues else set()
             if task_queue in active_queues:
                 return True
@@ -215,6 +232,9 @@ def _is_worker_alive(result: AsyncResult, timeout=None, retry_delay=0.1, retries
                          f'{pformat(queues)}\n'
                          f'Active queues: {pformat(active_queues)}')
 
+        elif state == SUCCESS:
+            return True  # Timing; possible if task state changed after we waited on it but before we got here
+
         else:
             logger.debug(f'Unknown state ({state} for task {task_name}; assuming task is alive.')
             return True
@@ -246,58 +266,62 @@ def wait_on_async_results(results,
                           max_wait=None,
                           callbacks: [WaitLoopCallBack]=tuple(),
                           sleep_between_iterations=0.05,
-                          check_task_worker_frequency=900,
-                          fail_on_worker_failures=3,
+                          check_task_worker_frequency=600,
+                          fail_on_worker_failures=7,
                           log_msg=True
                           ):
     if not results:
         return
 
-    max_trials = max_wait/sleep_between_iterations if max_wait else None
-    trials = 0
-
     if isinstance(results, AsyncResult):
         results = [results]
 
+    max_sleep = sleep_between_iterations * 20  # Somewhat arbitrary
     failures = []
+    start_time = time.monotonic()
+    last_callback_time = {callback.func: start_time for callback in callbacks}
     for result in results:
         logging_name = get_result_logging_name(result)
         if log_msg:
             logger.debug('-> Waiting for %s to complete' % logging_name)
+
         try:
-            worker_failures = 0
+            task_worker_failures = 0
+            last_dead_task_worker_check = time.monotonic()
             while not is_result_ready(result):
                 if RevokedRequests.instance().is_revoked(result):
                     break
-
                 _check_for_traceback_in_parents(result, timeout=30)
 
-                if max_trials and trials >= max_trials:
+                current_time = time.monotonic()
+                if max_wait and (current_time - start_time) > max_wait:
                     logging_name = get_result_logging_name(result)
                     raise WaitOnChainTimeoutError('Result ID %s was not ready in %d seconds' % (logging_name, max_wait))
 
-                time.sleep(sleep_between_iterations)
-
-                trials += 1
-
                 # callbacks
                 for callback in callbacks:
-                    if trials % (callback.frequency / sleep_between_iterations) == 0:
+                    if (current_time - last_callback_time[callback.func]) > callback.frequency:
                         callback.func(**callback.kwargs)
+                        last_callback_time[callback.func] = current_time
 
                 # Check for dead workers
                 if check_task_worker_frequency and fail_on_worker_failures and \
-                        round(trials % (check_task_worker_frequency / sleep_between_iterations)) == 0:
+                        (current_time - last_dead_task_worker_check) > check_task_worker_frequency:
                     alive = _is_worker_alive(result=result)
+                    last_dead_task_worker_check = current_time
                     if not alive:
-                        worker_failures += 1
+                        task_worker_failures += 1
                         logger.warning(f'Task {get_task_name_from_result(result)} appears to be a zombie.'
-                                       f' Failures: {worker_failures}')
-                        if worker_failures >= fail_on_worker_failures:
+                                       f' Failures: {task_worker_failures}')
+                        if task_worker_failures >= fail_on_worker_failures:
                             raise ChainInterruptedByZombieTaskException(task_id=str(result),
                                                                         task_name=get_task_name_from_result(result))
                     else:
-                        worker_failures = 0
+                        task_worker_failures = 0
+
+                time.sleep(sleep_between_iterations)
+                sleep_between_iterations = sleep_between_iterations * 1.01 \
+                    if sleep_between_iterations*1.01 < max_sleep else max_sleep  # Exponential backoff
 
             if result.state == REVOKED:
                 raise ChainRevokedException(task_id=str(result),
